/*
 * audina.h - part of Audina audiobook stretcher
 * Copyright (C) Bohdan R. Rau 2010-2011 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */


#ifndef ETHANAK_AUDINA_H
#define ETHANAK_AUDINA_H 1


#define MP3_DECODER_NAME "mpg123"


#define AUDINA_FMAT_AUTO 0
#define AUDINA_FMAT_SNDFILE 1
#define AUDINA_FMAT_MP3 2
#define AUDINA_FMAT_AVC 3

#define AUDINA_FEAT_PAUSES 1
#define AUDINA_FEAT_MAXPAUSE 2
#define AUDINA_FEAT_TEMPO 4

#define AUDINA_F_VUMETER 1
#define AUDINA_F_VERBOSE 2
#define AUDINA_F_DUMPF 4
#define AUDINA_F_SCRIPT 8
#define AUDINA_F_NOID3 0x10
#define AUDINA_F_TUNE 0x20
#define AUDINA_F_FORCEPAUSE 0x40

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

extern int input_convert_sndfile(char *filename);
extern int input_convert_mad(char *filename);
extern int input_convert_mpg123(char *filename);
extern int input_convert_avcodec(char *filename);
extern int set_input_samplerate(int samplerate);
extern int put_samples(float *samples, int nsamples);
extern int init_lame_encoder(char *fname);
extern int lame_finish(void);
extern int lame_encode(short *buf,int nsamples);
extern int init_twolame_encoder(char *fname);
extern int twolame_finish(void);
extern int twolame_encode(short *buf,int nsamples);
extern int encoder_init(char *fname);
extern int encoder_finish(void);
extern int ao_stop();
extern int ao_play_samples(short *samples,int samp);
extern int init_ao_player(void);
extern int init_amrwb_encoder(char *fname);
extern int encode_amrwb(short *buf,int nsamples);
extern int finish_amrwb(void);

extern int init_wave_encoder(char *fname,int raw);
extern int wave_finish(void);
extern int wave_encode(short *buf,int nsamples);

extern double tempo;
extern int use_feats;
extern int max_pause;
extern double pause_length;
extern int set_type;
extern double density;
extern double mad_gain;
extern double pitch;
extern int special_flags;
extern int min_pause_frames;

extern double eqpole[3];
extern int eqband[2];
extern int use_equalizer;
extern double eqgain;

extern int output_samplefreq;
extern int output_freq_set;
extern int output_mp3_brate;

extern int use_contrast;
extern double contrast_level;

extern int samplefreq;
extern int libavc_audio_idx;
extern int64_t recording_time;
extern int part_time;
extern int64_t skip_time;
extern int64_t partial_samples;
extern int64_t sample_length;
extern int64_t current_samples;

extern char *title,*artist,*album;
extern int initial_track,current_track;

extern int resampler_quality;

char *id3_strdup(char *str);
char *id3_convert_out(char *str);

extern char *id3_input_enc;
extern char *id3_output_enc;

extern int input_is_raw;
extern int input_raw_channels;
extern int input_raw_type;
extern int input_raw_rate;
extern int output_padding;

#define RAW_ITYPE_S16 0
#define RAW_ITYPE_S24 1
#define RAW_ITYPE_S32 2
#define RAW_ITYPE_FLOAT 3

// #define SILENCE_LIMIT 0.0025
#define SILENCE_LIMIT 0.008

extern int use_pa;

#endif
