/*
 * avcodec.c - part of Audina audiobook stretcher
 * Copyright (C) Bohdan R. Rau 2010-2011 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */

#include "config.h"
#include "audina.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libavcodec/avcodec.h>
#include <libavformat/avformat.h>

#define FFPS_SIZE 4096

#ifdef AVCODEC_HAS_AVTA
#define AUDINA_AVC_AUDIO AVMEDIA_TYPE_AUDIO
#else
#define AUDINA_AVC_AUDIO CODEC_TYPE_AUDIO
#endif

static int ffmpeg_put(short *buf,int size,int channels)
{
    float fltbuf[FFPS_SIZE];
    int i,nsamp,ret;
    int samples=size/2;
    if (channels == 1) {
        while (samples > 0) {
            nsamp=samples;
            if (nsamp > FFPS_SIZE) nsamp=FFPS_SIZE;
            for (i=0;i<nsamp;i++) {
                fltbuf[i]=(mad_gain * buf[i])/32768.0;
            }
            ret=put_samples(fltbuf,nsamp);
            if (ret<0) return ret;
            buf+=nsamp;
            samples-=nsamp;
        }
        return 0;
    }
    samples=samples/2;
    while(samples > 0) {
        nsamp=samples;
        if (nsamp > FFPS_SIZE) nsamp=FFPS_SIZE;
        for (i=0;i<nsamp;i++) {
            fltbuf[i]=(mad_gain*(buf[2*i]+buf[2*i+1]))/65536.0;
        }
        ret=put_samples(fltbuf,nsamp);
        if (ret<0) return ret;
        buf+=2*nsamp;
        samples-=nsamp;
    }
    return 0;
}
int input_convert_avcodec(char *filename)
{
    AVFormatContext *p_DecFormatCtx=NULL;
    int             i_AudioStreamIdx;
    AVPacket        InputPacket;

    AVCodecContext  *p_InputCodecCtx;
    AVCodec         *p_InputCodec;
    AVFormatParameters params;

    int buffer_size = AVCODEC_MAX_AUDIO_FRAME_SIZE;

    static short inBuffer[AVCODEC_MAX_AUDIO_FRAME_SIZE];

    int ret,len,idx;
    int channels,rate;

    av_register_all();
    memset(&params,0,sizeof(params));
#ifdef AVFORMAT_HAS_OINPUT
	ret=avformat_open_input(&p_DecFormatCtx, filename, NULL, NULL);
#else
    ret=av_open_input_file(&p_DecFormatCtx, filename, NULL, 0, NULL);
#endif
    if (ret!=0) {
        return -1;
    }
    if ((ret = av_find_stream_info(p_DecFormatCtx)) < 0) {
        fprintf(stderr,"ffmpeg could not find CODEC parameters for %s", filename);
        return -1;
    }
    if (special_flags & AUDINA_F_VERBOSE)
#ifdef AVFORMAT_HAS_AVDF
     av_dump_format(p_DecFormatCtx, 0, filename, 0);
#else
     dump_format(p_DecFormatCtx, 0, filename, 0);
#endif
    i_AudioStreamIdx=-1;
    if (libavc_audio_idx >= 0) {
        if (libavc_audio_idx>=p_DecFormatCtx->nb_streams) {
            fprintf(stderr,"Audio stream index out of range 0..%d\n",p_DecFormatCtx->nb_streams-1);
            exit(1);
        }
        if (p_DecFormatCtx->streams[libavc_audio_idx]->codec->codec_type==AUDINA_AVC_AUDIO) {
            i_AudioStreamIdx = libavc_audio_idx;
        }
        else {
            fprintf(stderr,"Stream %d is not audio\n",libavc_audio_idx);
            exit(1);
        }
    }
    for(idx=0; idx<p_DecFormatCtx->nb_streams; idx++) {
        if(p_DecFormatCtx->streams[idx]->codec->codec_type==AUDINA_AVC_AUDIO &&
                i_AudioStreamIdx < 0) {
            i_AudioStreamIdx=idx;
        }
    }
    if (i_AudioStreamIdx < 0) {
        if (!errno) errno=ENOSYS;
        fprintf(stderr,"No audio track\n");
        return -1;
    }
    if (special_flags & 2) printf("Audio track index: %d\n",i_AudioStreamIdx);
    p_InputCodecCtx=p_DecFormatCtx->streams[i_AudioStreamIdx]->codec;
    p_InputCodec = avcodec_find_decoder(p_InputCodecCtx->codec_id);
    if(!p_InputCodec) {
        if (!errno) errno=ENOSYS;
        fprintf(stderr, "Unsupported codec!\n");
        return -1;
    }
#ifdef AVCODEC_HAS_OPEN2
    avcodec_open2(p_InputCodecCtx, p_InputCodec, NULL);
#else
    avcodec_open(p_InputCodecCtx, p_InputCodec);
#endif
    channels=p_InputCodecCtx->channels;
    if (channels != 1 && channels != 2) {
        fprintf(stderr,"Channels %d unsupported\n",channels);
        avcodec_close(p_InputCodecCtx);
        av_close_input_file(p_DecFormatCtx);
        errno=ENOSYS;
        return -1;
    }
    rate = p_InputCodecCtx->sample_rate;
    // track length
    //av_estimate_timings(p_DecFormatCtx);
    sample_length=rate * (p_DecFormatCtx->duration / AV_TIME_BASE);

    if (skip_time) {
        int64_t timestamp=AV_TIME_BASE * (int64_t)skip_time;
        errno=0;
        if (av_seek_frame(p_DecFormatCtx,-1,timestamp,0)<0) {
            avcodec_close(p_InputCodecCtx);
            av_close_input_file(p_DecFormatCtx);
            if (!errno) errno=EINVAL;
            return -1;
        }
        sample_length -= rate * (int64_t)skip_time;

    }
    set_input_samplerate(rate);


    while(av_read_frame(p_DecFormatCtx, &InputPacket)>=0) {
        int data_size=buffer_size;
        if(InputPacket.stream_index!=i_AudioStreamIdx) {
            av_free_packet(&InputPacket);
            continue;
        }
#ifdef AVCODEC_HAS_AUDIO3
        len=avcodec_decode_audio3(p_InputCodecCtx,inBuffer,&data_size,&InputPacket);
#else
        len=avcodec_decode_audio2(p_InputCodecCtx,
                (int16_t*)inBuffer,
                &data_size,InputPacket.data,InputPacket.size);
#endif
        if (len<0) continue;
        ret=ffmpeg_put(inBuffer,data_size,channels);
        if (ret<0) {
            avcodec_close(p_InputCodecCtx);
            av_close_input_file(p_DecFormatCtx);
            return ret;
        }
    }
    avcodec_close(p_InputCodecCtx);
    av_close_input_file(p_DecFormatCtx);
    return 0;
}
