/*
 * File:   equalizer.h
 * Author: Marcin Kwiatkowski
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#ifndef EQUALISER_H
#define	EQUALISER_H

struct EQSTATE {
    // Filter #1 (Low band)
    double lf; // Frequency
    double f1p0; // Poles ...
    double f1p1;
    double f1p2;
    double f1p3;

    // Filter #2 (High band)

    double hf; // Frequency
    double f2p0; // Poles ...
    double f2p1;
    double f2p2;
    double f2p3;

    // Sample history buffer

    double sdm1; // Sample data minus 1
    double sdm2; //                   2
    double sdm3; //                   3

    // Gain Controls

    double lg; // low  gain
    double mg; // mid  gain
    double hg; // high gain

};

#ifdef __cplusplus
extern "C" {
#endif

void init_equaliser(struct EQSTATE* es, int lowfreq, int highfreq, int mixfreq);
double equalise(struct EQSTATE* es, double sample);
#ifdef __cplusplus
}
#endif

#endif	/* EQUALISER_H */

