/*
 * id3.c - part of Audina audiobook stretcher
 * Copyright (C) Bohdan R. Rau 2010-2011 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "audina.h"
#include <ctype.h>
#include <iconv.h>

char *id3_input_enc;
char *id3_output_enc;

static int encodecmp(char *str,char *pat)
{
    while(*str) {
        if (!isalnum(*str)) {
            str++;
            continue;
        }
        if (tolower(*str++)!=*pat++) return 0;
    }
    return !*pat;
}

static int get_unichar(char **str)
{
	int znak,n,m;
	if (!*str) return 0;
	znak=(*(*str)++) & 255;
	if (!(znak & 0x80)) return znak;
	if ((znak & 0xe0)==0xc0) n=1;
	else if ((znak & 0xf0)==0xe0) n=2;
    else if ((znak & 0xf8)==0xf0) n=3;
    else if ((znak & 0xfc)==0xf8) n=4;
    else if ((znak & 0xfe)==0xfc) n=5;
    else return -1;
	znak &= (0x3f >> n);
	while (n--) {
		m=*(*str)++ & 255;
		if ((m & 0xc0)!=0x80) return -1;
		znak=(znak<<6) | (m & 0x3f);
	}
	return znak;
}

static int aid3_tsb(char *src)
{
    int *utmp,*us,rc,n;
    char *cs;
    utmp=malloc(sizeof(int)*(strlen(src)+1));
    cs=src;
    us=utmp;
    rc=0;
    for (;;) {
        n=get_unichar(&cs);
        if (n<0) goto cleanup; // not utf8
        if (n>255) goto cleanup; // ugly encoding
        *us++=n;
        if (!n) break;
    }
    cs=src;us=utmp;
    while ((*cs++=*us++));
    rc=1;
cleanup:
    free(utmp);
    return rc;
}

static void aid3_udecode(char *src)
{
    int *utmp,*us,*bs,n,m,znak;
    char *cs,*dst;
    utmp=malloc(sizeof(int)*(strlen(src)+1));
    cs=src;
    us=utmp;
    for (;;) {
        n=get_unichar(&cs);
        if (n<0) goto cleanup; // not utf8
        *us++=n;
        if (!n) break;
    }
    // double encoded?
    us=utmp;
    bs=utmp;
    while (*us) {
        znak=*us++;
        if (znak>255) goto cleanup; // not 8-bit
        if (! (znak & 0x80)) {
            *bs++=znak;
            continue;
        }
        if ((znak & 0xe0)==0xc0) n=1;
        else if ((znak & 0xf0)==0xe0) n=2;
        else if ((znak & 0xf8)==0xf0) n=3;
        else if ((znak & 0xfc)==0xf8) n=4;
        else if ((znak & 0xfe)==0xfc) n=5;
        else goto cleanup;
        znak= znak & (0x3f >> n);
        while (n--) {
            m=*us++;
            if (m > 255) goto cleanup;
            if ((m & 0xc0) != 0x80) goto cleanup;
            znak=(znak << 6) | (m & 0x3f);
        }
        *bs++=znak;
	}
    *bs=0;
    for (us=utmp,dst=src;*us;us++) {
        if ((*us) < 0x80) {
            *dst++=*us;
        }
        else if ((*us) <=0x7ff) {
            *dst++=0xc0 | ((*us) >> 6);
            *dst++=0x80 | ((*us) & 0x3f);
        }
        else if ((*us) < 0x10000) {
            *dst++=0xe0 | ((*us) >> 12);
            *dst++=0x80 | (((*us) >> 6) & 0x3f);
            *dst++=0x80 | ((*us) & 0x3f);
        }
        else *dst++='?';
    }
    *dst=0;
cleanup:
    free(utmp);
    return;
}

char *id3_strdup(char *str)
{
    char *ostr;
    iconv_t ic;
    size_t len1,len2;
    char *ptr1,*ptr2;
    if (!id3_input_enc || encodecmp(id3_input_enc,"utf8")) {
        str=strdup(str);
        aid3_udecode(str);
        return str;
    }
    if (encodecmp(id3_input_enc,"latin1") ||
        encodecmp(id3_input_enc,"iso88591") ||
        encodecmp(id3_input_enc,"cp1252")) {
            return strdup(str);
    }
    str=strdup(str);
    if (!aid3_tsb(str)) return str; // something wrong?
    ic=iconv_open("UTF-8",id3_input_enc);
    if (ic == (iconv_t)-1) return str;
    len1=strlen(str);
    ptr1=str;
    len2=6*len1;
    ostr=malloc(len2+1);
    ptr2=ostr;
    iconv(ic,&ptr1,&len1,&ptr2,&len2);
    *ptr2=0;
    iconv_close(ic);
    free(str);
    return ostr;
}

#if 0
static int id3_is_latin1(char *str)
{
    for (;;) {
        int n=get_unichar(&str);
        if (n<0 || n>255) return 0;
        if (!n) break;
    }
    return 1;
}
#endif

char *id3_convert_out(char *str)
{
    char benc[256],*enc;
    char *ostr;
    iconv_t ic;
    size_t len1,len2;
    char *ptr1,*ptr2;
    
    if (!str || !*str) return str;

    if (!id3_output_enc) {
            aid3_tsb(str);
            return str;
    }
    if (!encodecmp(id3_output_enc,"utf8")) return str;
    if (encodecmp(id3_output_enc,"latin1") ||
        encodecmp(id3_output_enc,"iso88591") ||
        encodecmp(id3_output_enc,"cp1252")) {
        aid3_tsb(str);
        return str;
    }
    enc=id3_output_enc;

    if (strlen(enc)<200 && !strstr(enc,"//")) {
        sprintf(benc,"%s//translit",enc);
        enc=benc;
    }
    ic=iconv_open(enc,"UTF-8");
    if (ic == (iconv_t)-1) return NULL;
    len1=strlen(str);
    ptr1=str;
    len2=2*len1;
    ostr=malloc(len2+1);
    ptr2=ostr;
    iconv(ic,&ptr1,&len1,&ptr2,&len2);
    *ptr2=0;
    iconv_close(ic);
    return ostr;
}
