/*
 * lame.c - part of Audina audiobook stretcher
 * Copyright (C) Bohdan R. Rau 2010-2011 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */


#include "audina.h"
#include <stdio.h>
#include <lame/lame.h>

static FILE *mp3_file;
static lame_t lame_encoder;

#define LAME_BUFSIZE (8192 + 1024 + 7200)


int init_lame_encoder(char *fname)
{
	char *c;
    mp3_file=fopen(fname,"w+");
	if (!mp3_file) {
		perror(fname);
		return 0;
	}
	lame_encoder=lame_init();
	lame_set_num_channels(lame_encoder,1);
	lame_set_in_samplerate(lame_encoder,output_samplefreq);
	lame_set_mode(lame_encoder,MONO);
	lame_set_brate(lame_encoder,output_mp3_brate);
    if (artist) {
        c=id3_convert_out(artist);
        if (c) artist=c;
    }
    if (album) {
        c=id3_convert_out(album);
        if (c) album=c;
    }
    if (title) {
        c=id3_convert_out(title);
        if (c) title=c;
    }
    
    if (!(special_flags & AUDINA_F_NOID3) && (artist || title || album || current_track)) {
        id3tag_add_v2(lame_encoder);
        id3tag_pad_v2(lame_encoder);
		if (artist) id3tag_set_artist(lame_encoder,artist);
		if (album) id3tag_set_album(lame_encoder,album);
		if (title) id3tag_set_title(lame_encoder,title);
		if (current_track) {
            char trk[16];
            sprintf(trk,"%d",current_track);
            id3tag_set_track(lame_encoder,trk);
            current_track++;
        }
		id3tag_set_genre(lame_encoder,"101");
    }

	int ret=lame_init_params(lame_encoder);
	if (ret<0) {
		fclose(mp3_file);
		errno=ENOSYS;
        return -1;
	}
	return 0;
}

int lame_encode(short *buf,int nsamples)
{
	unsigned char outbuf[LAME_BUFSIZE];
	int n;
	n=lame_encode_buffer(lame_encoder,buf,buf,nsamples,outbuf,LAME_BUFSIZE);
	if (n<0) return -1;
	if (n) fwrite(outbuf,1,n,mp3_file);
	return 0;
}

int lame_finish(void)
{
	unsigned char outbuf[LAME_BUFSIZE];
	int n;
	n=lame_encode_flush(lame_encoder,outbuf,LAME_BUFSIZE);
	if (n<0) {
		fclose(mp3_file);
		return n;
	}
	if (n) fwrite(outbuf,1,n,mp3_file);
	fflush(mp3_file);
	lame_mp3_tags_fid(lame_encoder,mp3_file);
	lame_close(lame_encoder);
	fclose(mp3_file);
	return 0;
}
