/*
 * sf.c - part of Audina audiobook stretcher
 * Copyright (C) Bohdan R. Rau 2010-2011 <ethanak@polip.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write see:
 *               <http://www.gnu.org/licenses/>.
 */


#include "audina.h"
#include <string.h>
#include <sndfile.h>

int input_convert_sndfile(char *filename)
{
    SF_INFO info;
    SNDFILE *file;
    float buffer[8192];
    int nsamp,rc;
    int formats[]={SF_FORMAT_PCM_16 | SF_ENDIAN_LITTLE,SF_FORMAT_PCM_24  | SF_ENDIAN_LITTLE,SF_FORMAT_PCM_32 | SF_ENDIAN_LITTLE,SF_FORMAT_FLOAT};
    memset(&info,0,sizeof(info));
    if (input_is_raw) {
		info.samplerate=input_raw_rate;
		info.channels=input_raw_channels;
		info.format=SF_FORMAT_RAW | formats[input_raw_type];
	}
    if (!strcmp(filename,"-")) {
		file=sf_open_fd(0,SFM_READ,&info,0);
	}
	else {
		file=sf_open(filename,SFM_READ,&info);
	}
    if (!file) return -1;
    if (info.channels < 1 || info.channels > 2) {
        sf_close(file);
        fprintf(stderr,"%d channels not supported\n",info.channels);
        errno=ENOSYS;
        return -1;
    }
    sample_length=info.frames;
    if (sample_length < 0) sample_length=0;
    if (skip_time > 0) {
        sf_count_t skip_frames=info.samplerate * (sf_count_t)skip_time;
        sf_count_t seek_rc=sf_seek(file,skip_frames,SEEK_SET);
        if (seek_rc < 0) {
            sf_close(file);
            return -1;
        }
        sample_length-=seek_rc;
        skip_time=seek_rc/info.samplerate;
    }
    current_samples=0;
    rc=set_input_samplerate(info.samplerate);
    
    if (rc) {
        sf_close(file);
        return -1;
    }
    for (;;) {
        int i;
        nsamp=sf_readf_float(file,buffer,8192);
        if (nsamp<=0) break;
        if (info.channels == 2) {
            nsamp=nsamp/2;
            for (i=0;i<nsamp;i++) {
                buffer[i]=mad_gain * (buffer[2*i]+buffer[2*i+1])/2;
            }
        }
        else {
            for (i=0;i<nsamp;i++) buffer[i] = mad_gain * buffer[i];
        }
        current_samples+=nsamp;
        rc=put_samples(buffer,nsamp);
        if (rc<0) {
            sf_close(file);
            return -1;
        }
    }
    rc=put_samples(NULL,0);
    sf_close(file);
    return 0;
}

static SNDFILE *output_sf;
 
int init_wave_encoder(char *fname,int raw)
{
    SF_INFO info;
    memset(&info,0,sizeof(info));
    info.frames=0;
    info.samplerate=output_samplefreq;
    info.channels=1;
    info.format=(raw?SF_FORMAT_RAW:SF_FORMAT_WAV) | SF_FORMAT_PCM_16 | SF_ENDIAN_LITTLE;
    if (!strcmp(fname,"-")) {
        output_sf=sf_open_fd(1,SFM_WRITE,&info,0);
    }
    else {
        output_sf=sf_open(fname,SFM_WRITE,&info);
    }
    if (!output_sf) {
        perror(fname);
        return -1;
    }
    return 0;
}

int wave_encode(short *buf,int nsamples)
{
    if (sf_write_short(output_sf,buf,nsamples)!=nsamples) {
        perror("Wave write");
        return -1;
    }
    return 0;
}

int wave_finish()
{
    sf_close(output_sf);
    return 0;
}
